@@ -1,43 +1,44 @@
-# $Id$
+# $Id: Counting.pm 919 2008-08-07 19:13:23Z gene $
 package Math::Counting;
+our $VERSION = '0.08';
+
 use strict;
 use warnings;
-use base 'Exporter';
-use vars qw( @EXPORT_OK @EXPORT %EXPORT_TAGS $VERSION );
 use Carp;
-use Math::BigInt; #try => 'GMP';
+use base 'Exporter';
+use vars qw( @EXPORT %EXPORT_TAGS );
+use Math::BigInt;
 
-$VERSION = 0.0704;
-@EXPORT_OK = ();
 @EXPORT = qw(
-    f  factorial    bfact
-    P  permutation  bperm
-    C  combination  bcomb
+    factorial   bfact
+    permutation bperm
+    combination bcomb
 );
 %EXPORT_TAGS = (
-    short => [qw( f P C )],
-    long  => [qw( factorial permutation combination )],
-    big   => [qw( bfact bperm bcomb )],
+    student => [qw( factorial permutation combination )],
+    big =>     [qw( bfact     bperm       bcomb )],
 );
 
-sub f { factorial(@_) }
+# The algorithmically elegant way:
 sub factorial {
     my( $n ) = @_;
+    return unless defined $n && $n =~ /^\d+$/;
     my $product = 1;
     while( $n > 0 ) {
         $product *= $n--;
     }
     return $product;
 }
+
 # The right way to do it:
 sub bfact {
     my $n = Math::BigInt->new( shift );
     return $n->bfac();
 }
 
-sub P { permutation(@_) }
 sub permutation {
     my( $n, $r ) = @_;
+    return unless defined $n && $n =~ /^\d+$/ && defined $r && $r =~ /^\d+$/;
     my $product = 1;
     while( $r > 0 ) {
         $product *= $n--;
@@ -45,6 +46,7 @@ sub permutation {
     }
     return $product;
 }
+
 sub bperm {
     my( $n, $r ) = @_;
     $n = Math::BigInt->new( $n );
@@ -53,9 +55,9 @@ sub bperm {
     return $n->bfac() / $r->bfac();
 }
 
-sub C { combination(@_) }
 sub combination {
     my( $n, $r ) = @_;
+    return unless defined $n && $n =~ /^\d+$/ && defined $r && $r =~ /^\d+$/;
     my $product = 1;
     while( $r > 0 ) {
         $product *= $n--;
@@ -63,6 +65,7 @@ sub combination {
     }
     return $product;
 }
+
 sub bcomb {
     my( $n, $k ) = @_;
     $n = Math::BigInt->new( $n );
@@ -81,38 +84,32 @@ Math::Counting - Combinatorial counting operations
 
 =head1 SYNOPSIS
 
-  ## The algorithm-student versions:
-
-  use Math::Counting ':long';
+  use Math::Counting ':student';
   printf "Given n=%d and r=%d:\nFact=%d\nPerm=%d\nComb=%d\n",
     $n, $r, factorial($n), permutation($n, $r), combination($n, $r);
 
-  use Math::Counting ':short';
-  printf "Given n=%d and r=%d:\nF=%d\nP=%d\nC=%d\n",
-    $n, $r, f($n), P($n, $r), C($n, $r);
-
-  ## The "Right way to do it":
-
   use Math::Counting ':big';
   printf "n=%d, r=%d:\nBig F=%d\n Big P=%d\nBig C=%d\n",
     $n, $r, bfact($n), bperm($n, $r), bcomb($n, $r);
 
 =head1 DESCRIPTION
 
-This package computes the factorial, number of (n,r) permutations and
-(n,r) combinations using either L<Math::BigInt> and the technique of
-"tail call elimination" (TCE).
+Compute the numerical factorial, number of n,r permutations and number
+of n,r combinations using "infinate precision arithmetic" and also the
+technique of "tail call elimination" as detailed in
+B<Higher Order Perl> and based on the algorithms in
+B<Mastering Algorithms with Perl>.
 
 No functions are exported by default.
 
 =head1 FUNCTIONS
 
-=head2 f, factorial
+=head2 factorial
 
-  $f = f($n);
   $f = factorial($n);
 
-Return the number of arrangements of B<n> with TCE.
+Return the number of arrangements of B<n> according to the "student"
+version using real arithmetic.
 
 =head2 bfact
 
@@ -121,13 +118,13 @@ Return the number of arrangements of B<n> with TCE.
 Return the value of the L<Math::BigInt/bfact> function, which is the
 "Right Way To Do It."
 
-=head2 P, permutation
+=head2 permutation
 
-  $p = P($n, $r);
   $p = permutation($n, $r);
 
 Return the number of arrangements of B<r> elements drawn from a set of
-B<n> elements with TCE.  (B<nPn> is the same as B<n!>.)
+B<n> elements.  B<nPn> is the same as B<n!>.  This function employs
+the "student" version using real arithmetic.
 
 =head2 bperm
 
@@ -135,14 +132,13 @@ B<n> elements with TCE.  (B<nPn> is the same as B<n!>.)
 
 Return the C<Math::BigInt> computation: B<n!/(n-r)!>.
 
-=head2 C, choose, combination
+=head2 combination
 
-  $c = C($n, $r);
-  $c = choose($n, $r);
   $c = combination($n, $r);
 
 Return the number of ways to choose B<r> elements from a set of B<n>
-elements with TCE.
+elements.  This function employs the "student" version using real
+arithmetic.
 
 =head2 bcomb
 
@@ -181,11 +177,11 @@ L<http://en.wikipedia.org/wiki/Combination>
 
 =head1 AUTHOR
 
-Not me. I am but a pebble on the beach...
+Gene Boggs E<lt>gene@cpan.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2007 Gene Boggs All Rights Reserved
+Copyright 2005-2007 Gene Boggs All Rights Reserved
 
 You may use this package under the same terms as Perl itself.
 
